<?php
// $Id: datasync.hosts.inc,v 1.1.2.7 2009/04/24 01:56:30 andrewlevine Exp $

function datasync_hostname() {
  exec("hostname", $current);
  trim($current[0]);
  return $current[0];
}

//Verifies if a process is running in linux
function datasync_process_running($pid) {
  exec("ps $pid", $state);
  return(count($state) >= 2);
}

//fetch nice array of hosts
function datasync_get_hosts($get_host = NULL) {
  static $ret = FALSE;
  if ($ret === FALSE) {
    $hosts = ds_variable_get('datasync_hosts', '');
    //get rid of non-unix line breaks
    $hosts = str_replace("\r\n", "\n", $hosts);
    $hosts = str_replace("\r", "\n", $hosts);
    $hosts = explode("\n\n", $hosts);
    
    $ret = array();
    $one_host_runs_post_hook = FALSE;
    foreach ($hosts as $host) {
      //clean up and take hostname
      $host = explode("\n", $host);
      array_map('trim', $host);
      foreach ($host as $key => $line) {
        if (empty($line)) {
          unset($host[$key]);
        }
      }
      
      $hostname = array_shift($host);
      
      //save values
      $data = implode("\n", $host);
      $ret[$hostname] = empty($host) ? array() : datasync_parse_host_info($data);
      
      //record if at least one host runs the post hook
      if ($ret[$hostname]['run_post_hook']) {
        $one_host_runs_post_hook = TRUE;
      }
      else {
        $ret[$hostname]['run_post_hook'] = FALSE;
      }
    }
    
    //if no-one is running the post hook, then everyone has to just to be safe
    if (!$one_host_runs_post_hook) {
      foreach ($ret as $hostname => $settings) {
        $ret[$hostname]['run_post_hook'] = TRUE;
      }
    }
  }
  return (($get_host === NULL) ? $ret : $ret[$get_host]);
}

function datasync_parse_host_info($data) {
  //ripped from drupal_parse_info_file because that
  //function assumes a file
  $info = array();
  if (preg_match_all('
    @^\s*                           # Start at the beginning of a line, ignoring leading whitespace
    ((?:
      [^=;\[\]]|                    # Key names cannot contain equal signs, semi-colons or square brackets,
      \[[^\[\]]*\]                  # unless they are balanced and not nested
    )+?)
    \s*=\s*                         # Key/value pairs are separated by equal signs (ignoring white-space)
    (?:
      ("(?:[^"]|(?<=\\\\)")*")|     # Double-quoted string, which may contain slash-escaped quotes/slashes
      (\'(?:[^\']|(?<=\\\\)\')*\')| # Single-quoted string, which may contain slash-escaped quotes/slashes
      ([^\r\n]*?)                   # Non-quoted string
    )\s*$                           # Stop at the next end of a line, ignoring trailing whitespace
    @msx', $data, $matches, PREG_SET_ORDER)) {
    foreach ($matches as $match) {
      // Fetch the key and value string
      $i = 0;
      foreach (array('key', 'value1', 'value2', 'value3') as $var) {
        $$var = isset($match[++$i]) ? $match[$i] : '';
      }
      $value = stripslashes(substr($value1, 1, -1)) . stripslashes(substr($value2, 1, -1)) . $value3;

      // Parse array syntax
      $keys = preg_split('/\]?\[/', rtrim($key, ']'));
      $last = array_pop($keys);
      $parent = &$info;

      // Create nested arrays
      foreach ($keys as $key) {
        if ($key == '') {
          $key = count($parent);
        }
        if (!isset($parent[$key]) || !is_array($parent[$key])) {
          $parent[$key] = array();
        }
        $parent = &$parent[$key];
      }

      // Handle PHP constants
      if (defined($value)) {
        $value = constant($value);
      }

      // Insert actual value
      if ($last == '') {
        $last = count($parent);
      }
      $parent[$last] = $value;
    }
  }
  return $info;
}

/**
 * Returns host-specific job settings
 * including phase settings
 * 
 * @return 
 *   array of job settings
 */
function datasync_host_job_settings($chost) {
  static $ret = FALSE;
  if ($ret === FALSE) {
    //set up $host_jobs array to hold host-specific job settings
    $host_jobs = array();
    $host_settings = datasync_get_hosts($chost);
    if (isset($host_settings['jobs']) && is_array($host_settings['jobs']) && count($host_settings['jobs'])) {
      $host_jobs = array_map('trim', $host_settings['jobs']);
      $host_jobs = array_flip($host_jobs);
    }
    if ($host_has_phases && is_array($host_settings['phases']) && count($host_settings['phases'])) {
      foreach ($host_jobs as $type => $nothing) {
        if (isset($host_settings['phases'][$type]) && is_array($host_settings['phases'][$type]) 
          && count($host_settings['phases'][$type])) {
          foreach ($host_settings['phases'][$type] as $phase) {
            $host_jobs[$type][] = $phase;
          }
        } 
      }
    }
    $ret = $host_jobs;
  }
  return $ret;
}

function datasync_host_proc_settings($chost) {
  static $ret = FALSE;
  if ($ret === FALSE) {
    $host_settings = datasync_get_hosts($chost);
    //set up host_procs array to hold-specific process settings
    $ret = (isset($host_settings['procs']) && is_array($host_settings['procs']) && count($host_settings['procs'])) ? $host_settings['procs'] : array();
  }
  return $ret;
}
