<?php
// $Id: datasync_run.php,v 1.1.2.13 2009/07/24 18:41:00 andrewlevine Exp $

//die if we are not on the command line
if (!$argc) {
  exit();
}

//no need to chdir, we are already at drupal home

$args = $_SERVER['argv'];
$host = $args[1];
global $datasync_debug;
if ($args[2] && $args[2] == 'debug') {
  $datasync_debug = TRUE;
} 
else {
  $datasync_debug = FALSE;
}


//bootstrap drupal
//hack in fake values for this global to trick drupal into bootstrapping
$GLOBALS['_SERVER']['SCRIPT_NAME'] = $host . '/cron.php';
require_once './includes/bootstrap.inc';
drupal_bootstrap(DRUPAL_BOOTSTRAP_FULL);


//no session
session_save_session(FALSE);
//make sure there is not time limit
set_time_limit(0);
//up memory_limit
$mem_lim = ds_variable_get('datasync_run_memory_limit', datasync_return_bytes(ini_get('memory_limit')));
ini_set('memory_limit', $mem_lim);

//fake login
global $user;
$tmp_user = $user;
$user = user_load(array('uid' => 1));


//we need a function to make sure we don't have database/forking issues
$db_name = db_set_active();
function datasync_remove_database_conn($db_name) {
  unset($GLOBALS['db_conns'][$db_name]);
  unset($GLOBALS['active_db']);
}


$start = time();

module_load_include('inc', 'datasync', 'datasync.job');
module_load_include('inc', 'datasync', 'datasync.hosts');

datasync_debug("Drupal bootstrapped and necessary DataSync files included...\n");

set_error_handler('datasync_error_handler');

$chost = datasync_hostname();
$wait = 0;
$max_wait = ds_variable_get('datasync_max_wait', 20);

//get connection info to create new DB resource links
global $db_url;

//create process children list for bookkeeping
$children = array();
$children_jid = array();
$children_count = array();


//EVERYTHING FROM HERE TO THE while(TRUE) loop
//IS USED TO LOAD HOST-SPECIFIC CONFIGURATION
$host_settings = datasync_get_hosts($chost);

datasync_debug("Host settings loaded");
datasync_debug(print_r($host_settings, TRUE));

$max_procs = isset($host_settings['max_procs']) ? (int)$host_settings['max_procs'] : variable_get('datasync_default_max_procs', 5);
$host_jobs = datasync_host_job_settings($chost);
$procs = datasync_host_proc_settings($chost);
$host_has_phases = isset($host_settings['phases']);
$host_runs_post_hook = isset($host_settings['run_post_hook']) && $host_settings['run_post_hook'];

//TODO validate settings


$job_types = datasync_job_types(TRUE); // parameter is $enabled_only
if (count($host_jobs)) {
  //if host_jobs is defined, delete all enabled types that are not in host-specific definition
  foreach ($job_types as $type => $def) {
    if (!isset($host_jobs[$type])) {
      unset($job_types[$type]);
      $job_types[$type]['phases'] = NULL;
    }
  }
  if ($host_has_phases) {
    //if phases in host_jobs is defined add appropriate phases and remove jobs without phases
    foreach ($host_jobs as $type => $phases) {
      if (isset($job_types[$type]) && is_array($phases) && count($phases)) {
        $job_types[$type]['phases'] = $phases;
      }
      else {
        unset($job_types[$type]);
      }
    }
  }
}

datasync_debug("Host-specific job settings loaded:");
datasync_debug(print_r($job_types, TRUE));

datasync_debug("Starting while loop.");

while (TRUE) {
/////////////START WHILE  

$jobs = array();
try {
  if ($host_has_phases && !empty($procs)) {
    //run take_jobs calls separately to get specific amounts of certain processess
    foreach ($procs as $job_type => $phases) {
      foreach ($phases as $phase => $num_procs) {
        //don't go over host+job/phase-specific number of procs
        $needed_procs = $num_procs - count($children_count[$job_type . '||' . '$phase']);
        $needed_procs = ($num_procs < 0) ? 0 : $num_procs;
        //don't go over total number of procs
        if ($max_procs <= array_sum($children_count)) {
          //already too many running on this host
          $needed_procs = 0;
        }
        else if ($needed_procs > ($max_procs - array_sum($children_count))) {
         $needed_procs = ($max_procs - array_sum($children_count));
        }
        
        if ($needed_procs >= 0) {
          $types_phases = array($job_type => $phase);
          $more_jobs = datasync_job::take($needed_procs, $types_phases);
          if (is_array($more_jobs)) {
            $jobs = array_merge($jobs, $more_jobs);
          }
        }
      }
    }
  }
  else {
    //since we don't worry about phase-specific process counting, our take_jobs call is a lot simpler
    $types_phases = array();
    foreach ($job_types as $job_type => $def) {
      $types_phases[$job_type] = NULL;
      if ($host_has_phases) {
        $types_phases[$job_type] = $def['phases'];
      }
    }
    $needed_procs = $max_procs - array_sum($children_count);
    datasync_debug("$needed_procs available procs...");
    
    if ($needed_procs > 0) {
      datasync_debug("Asking for jobs with these types/phases:");
      datasync_debug(print_r($types_phases, TRUE));
      $jobs =  datasync_job::take($needed_procs, $types_phases);
    }
  }
  
  datasync_debug(count($jobs) . " jobs taken.");
  
  //retrieve basic info on jobs so we can use them without a database connection
  foreach ($jobs as $key => $job_child) {
    $jobs[$key]->construct_from_db();
  }
    
  $pid = -1;
  
  $job = NULL;
  
  //initially unset DB resource to prevent MySQL gone away problems
  datasync_remove_database_conn($db_name);
  
  foreach ($jobs as $job_child) {
    $pid = pcntl_fork();
    if ($pid == -1) {
      //error with forking, skip
      continue;
    }
    else if ($pid == 0) {
      //children break and start processing jobs
      
      //cloning so children are working on a totally different object
      $job = clone($job_child);
      break;
    }
    else {
      //parents keep making children
      
      datasync_debug("$pid forked to run job.");
      
      $job = $job_child;
      //keep track of child
      $children[$pid] = $job->type . '||' . $job->phase;
      $children_jid[$pid] = $job->job_id;
      $children_count[$children[$pid]]++;
    }
  }
  
  if ($pid == 0) {
    //children should run job and die
    
    //create new db connection for child
    db_set_active($db_name);
    datasync_debug("Created new DB connection for " . $GLOBALS['active_db'] . " child...");

    datasync_debug("Child for job $job->job_id is running...");

    if ($job) {
      $handler = datasync_load_job_type($job->job_id);
      $job = new $handler($job->job_id);
      if ($job->phase_proceed()) {
        $job->do_phase(); // actually runs phase
        datasync_debug("Child did phase...");
        
        $job->advance_phase(); // advances job status to next phase
        datasync_debug("Child advanced phase...");
      }
      else {
        $job->postpone_phase();
        datasync_debug("Child postponed phase...");
      }
    }
    //datasync_debug('trans_status: ' .print_r($job->transaction_status,TRUE));
    datasync_debug("Child exiting...");
    exit(0);
  }
  else {
    //parent continues
    
    if ($host_runs_post_hook) {
      //create one last child to run the post job hook
      $pid = pcntl_fork();
      if ($pid == -1) {
        //error with forking, continue
      }
      else if ($pid == 0) {
        //create db resource for post_job child
        db_set_active($db_name);
        //child invokes the post hook on all modules and exits
        //NOTE: do NOT NOT NOT NOT implement this hook for anything but necessary small
        //datasync-related cleanup that cannot be run in a regular datasync job. It was
        //originally added for the scheduler which needs time guarantees.
        module_invoke_all('datasync_post_jobs');
        datasync_debug("Invoked datasync_post_jobs hooks");
        exit(0);
      }
      else {
        //parent breaks out of the 2 curly braces below and continues
      }
    }
    //create db resource for parent again
    db_set_active($db_name);
    if ($GLOBALS['active_db']) {
      datasync_debug("Created new DB connection " . $GLOBALS['active_db'] . " for parent...");
    }
    else {
      datasync_debug("Failed to create DB connection for parent");
    }
    
    datasync_debug("Parent waiting for children...");
     
    //kill zombie children
    foreach ($children as $cpid => $info) {
      $pstatus = NULL;
      if (pcntl_waitpid($cpid, $pstatus, WUNTRACED | WNOHANG)) {
        datasync_debug("Parent sees child with PID $cpid and job_id $children_jid[$cpid] has completed...");
        //datasync_debug("children_count: " . print_r($children_count, TRUE));
        //datasync_debug("children: " . print_r($children, TRUE));
        //datasync_debug("children_jid: " . print_r($children_jid, TRUE));
        //datasync_debug("COUNT: " . $children_count[$children[$cpid]]);
        $children_count[$children[$cpid]]--;
        unset($children[$cpid]);
        unset($children_jid[$cpid]);
      }
    }
    
    //if there were no jobs and no wait then start waiting at 1s
    if (count($jobs) == 0 && $wait == 0) {
      $wait = 1;
    }
    else if (count($jobs)){
      $wait = 0;
    }
    
    $mem_padding = ds_variable_get('datasync_host_memory_padding', 10485760); //10M in bytes
    
    if (
        ((time()-$start) > 3600) //kill script every hour
    ||  (memory_get_usage() > ($mem_lim - $mem_padding)) //kill script if we are almost out of memory
    ||  (ds_variable_get("datasync_host_kill_$chost", 0)) //kill script if we are sent a signal
       ) {
      ds_variable_set("datasync_host_kill_$chost", 0);
      exit(0);
    }
    
    //exponential backoff
    datasync_debug("Sleeping for $wait seconds...");
    sleep($wait);
    $wait = 2*$wait;
    $wait = min($wait, $max_wait);
  }
}
catch (DataSyncException $e) {
  //datasync_log_failure($e->datasync_failure_type(), (string)$e);
  $exit_code = (!$e->getCode() ? 1 : $e->getCode());
  exit($exit_code);
}


/////////////END WHILE
}

$user = $tmp_user;


