<?php
/**
 * Page callback for scheduler addition page.
 * @return 
 *   HTML string
 */
function datasync_schedule_page() {
  drupal_add_css(drupal_get_path('module', 'datasync_scheduler') . '/datasync_scheduler.css');
  ctools_include('form');
  
  $output = '';
  $output .= '<div id="datasync_schedule_page">';
  $output .= '<h2>' . t('Add a scheduled job') . '</h2>';
  $form_state = array();
  
  $output .= ctools_build_form('datasync_schedule_job_form', $form_state);

  $output .= '<h2>' . t('Current scheduled jobs') . '</h2>';
  //form function supports retrieving all data in one query
  $result = db_query("SELECT * FROM {datasync_scheduler_additions} ORDER BY schedule_id ASC");
  $i = 0;
  while ($row = db_fetch_object($result)) {
    $form_state = array('args' => array($row->schedule_id, $row, $row->job_type));
    $output .= ctools_build_form('datasync_schedule_job_form_' . $i, $form_state);
    $i++;
  }
  $output .= '</div>';
  return $output;
}

function datasync_schedule_job_form($form_state, $page_index = 0, $schedule_id = NULL, $schedule_data = NULL, $job_type = NULL) {
  $form = array();
  $form['#theme'] = 'datasync_schedule_job_form';
  //set action manually so ajax pages return the right action
  $form['#action'] = '/admin/settings/datasync/schedule';
  $form['#attributes'] = array('class' => 'datasync-schedule-new');
  if ($schedule_id && ctype_digit($schedule_id)) {
    $form['#attributes'] = array('class' => 'datasync-schedule-' . $schedule_id);
  }
  
  $defaults = array();
  
  //If we have default values for this scheduled addition, set them
  if ($schedule_id != NULL && ctype_digit($schedule_id)) {
    if ($schedule_data) {
      $defaults = (array)$schedule_data;
    }
    else {
      $result = db_query("SELECT * FROM {datasync_scheduler_additions} WHERE schedule_id=%d", $schedule_id);
      $defaults = db_fetch_array($result);
    }
 
    //convert days/hours/minutes/seconds select boxes into one interval integer
    $defaults['days'] = (int)($defaults['add_interval'] / 86400); //86400 == seconds in 1 day
    $defaults['hours'] = (int)(($defaults['add_interval'] % 86400) / 3600); // 3600 == seconds in 1 hour
    $defaults['minutes'] = (int)(($defaults['add_interval'] % 3600) / 60); // 60 == seconds in 1 minute
    $defaults['seconds'] = (int)($defaults['add_interval'] % 60);
    
    //button text
    $defaults['submit'] = t('Save');
    $defaults['delete'] = t('Delete');
    
    //text for display on form of last time the job ran
    $form['#last_add'] = $defaults['last_add'] == 0 ? t('This schedule has not added a job to the queue yet.') : 
      t('This schedule last added a job to the queue @time ago', array('@time' => format_interval(time() - $defaults['last_add'], 6)));
    //submit function needs this data
    $form['#schedule_id'] = $schedule_id;
  }
  //set job type if it was specified
  if ($job_type) {
    $defaults['job_type'] = $job_type;
  }
  $options = array();
  foreach (datasync_job_types(TRUE) as $key => $def) {
    $options[$key] = $def['human_name'];
  }
  //we need to know the default job type in advance to do
  //custom form magic, so set it
  if (!$defaults['job_type']) {
    $available_job_types = array_keys($options);
    $defaults['job_type'] = $available_job_types[0];
  }

  $form['job_type'] = array(
    '#name' => 'datasync_scheduler_job_type',
    '#title' => t('Job type'),
    '#type' => 'select',
    '#options' => $options,
    '#description' => t('Type of job you want to add'),
    '#default_value' => $defaults['job_type'],
    '#attributes' => array('class' => 'ctools-use-ajax-onchange'),
  );
  
  //setup for ctools ajax functions
  ctools_add_js('ajax-responder');
  ctools_include('ajax');
  $url = url("datasync_scheduler/ajax/schedule_form");
  if ($schedule_id) {
    $url .= '/' . urlencode("$schedule_id|$page_index");
  }
  else {
    $url .= '/none';
  }
  ctools_ajax_associate_url_to_element($form, $form['job_type'], $url);
  
  $form['days'] = array(
    '#title' => t('Days'),
    '#type' => 'select',
    '#options' => drupal_map_assoc(range(0, 365)),
    '#default_value' => $defaults['days'] ? $defaults['days'] : FALSE,
  );
  $form['hours'] = array(
    '#title' => t('Hours'),
    '#type' => 'select',
    '#options' => drupal_map_assoc(range(0, 23)),
    '#default_value' => $defaults['hours'] ? $defaults['hours'] : FALSE,
  );
  $form['minutes'] = array(
    '#title' => t('Minutes'),
    '#type' => 'select',
    '#options' => drupal_map_assoc(range(0, 59)),
    '#default_value' => $defaults['minutes'] ? $defaults['minutes'] : FALSE,
  );
  $form['seconds'] = array(
    '#title' => t('Seconds'),
    '#type' => 'select',
    '#options' => drupal_map_assoc(range(0, 59)),
    '#default_value' => $defaults['seconds'] ? $defaults['seconds'] : FALSE,
  );
  
  $form['#custom_add_data'] = FALSE;
  //check if we have a custom add function
  module_load_include('inc', 'datasync', 'datasync.job');
  
  $type_handler = datasync_load_job_type($form['job_type']['#default_value']);
  //add any custom form stuff defined in plugin modules
  if ($type_handler && ($form_elements = module_invoke_all('datasync_scheduler_' . $type_handler. '_custom_schedule_form', $schedule_id, $schedule_data))) {
    $form['add_data'] = array(
      '#type' => 'fieldset',
      '#title' => t('Settings'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      '#tree'
    );

    $form['add_data'] = array_merge($form['add_data'], $form_elements);
    //note if we have a custom add data element
    $form['#custom_add_data'] = TRUE;
    $form['#datasync_type_handler'] = $type_handler;
  }
  
  $form['advanced'] = array(
    '#type' => 'fieldset',
    '#title' => t('Advanced'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#weight' => 10,
  );
  
  $form['advanced']['php_check'] = array(
    '#title' => t('PHP check'),
    '#type' => 'textarea',
    '#description' => t('Only schedule this job when this PHP code returns TRUE. Do not use @php_tags tags.', array('@php_tags' => '<?php ?>')),
    '#default_value' => $defaults['php_check'] ? $defaults['php_check'] : FALSE,
  );
  if (!$form['#custom_add_data']) {
    $form['advanced']['add_data'] = array(
      '#title' => t('Extra data'),
      '#type' => 'textarea',
      '#description' => t('When adding a job, the returned value will be used as extra data. Do not use @php_tags tags.', array('@php_tags' => '<?php ?>')),
      '#default_value' => $defaults['add_data'] ? $defaults['add_data'] : FALSE,
    );
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => $defaults['submit'] ? $defaults['submit'] : t('Add scheduled job'),
    '#submit' => array('datasync_schedule_job_form_submit'),
    '#validate' => array('datasync_schedule_job_form_validate'),
  );
  
  if ($defaults['delete']) {
    $form['delete'] = array(
      '#type' => 'submit',
      '#value' => $defaults['delete'],
      '#submit' => array('datasync_schedule_job_form_delete'),
    );
  }
  return $form;
}

function datasync_schedule_job_form_validate($form, $form_state) {
  $valid_interval = datasync_scheduler_interval_validate($form, $form_state);
  if (!$valid_interval) {
    form_set_error('', t("You must select some interval for job addition (can't all be 0)"));
  }

  if ($form['#custom_add_form']) {
    module_invoke_all('datasync_scheduler_' . $form['#datasync_type_handler']. '_custom_schedule_validate', $form, $form_state);
  }
}

function datasync_schedule_job_form_submit($form, $form_state) {
  $values = $form_state['values'];
  $add_interval = $values['days'] * 86400 + $values['hours'] * 3600 + $values['minutes'] * 60 + $values['seconds'];

  //fetch data for custom add form
  if ($form['#custom_add_data']) {
   $custom_add_structure = module_invoke_all('datasync_scheduler_' . $form['#datasync_type_handler']. '_custom_schedule_submit', $form, $form_state);
  }

  if (!isset($form['#schedule_id'])) {
    $schedule = array(
      'job_type' => $values['job_type'],
      'add_interval' => $add_interval,
      'last_add' => 0,
      'php_check' => $values['php_check'],
      'add_data' => $form['#custom_add_data'] ? serialize($custom_add_structure) : $values['add_data'],
      'custom_add_data' => (int)$form['#custom_add_data'],
    );
    drupal_write_record('datasync_scheduler_additions', $schedule);
  }
  else if (ctype_digit($form['#schedule_id'])) {
    $schedule = array(
      'schedule_id' => $form['#schedule_id'],
      'job_type' => $values['job_type'],
      'add_interval' => $add_interval,
      'php_check' => $values['php_check'],
      'add_data' => $form['#custom_add_data'] ? serialize($custom_add_structure) : $values['add_data'],
      'custom_add_data' => (int)$form['#custom_add_data'],
    );
    drupal_write_record('datasync_scheduler_additions', $schedule, 'schedule_id');
  }
}

function datasync_schedule_job_form_ajax($info = FALSE) {
  module_load_include('inc', 'datasync', 'datasync.job');
  ctools_include('ajax');
  ctools_include('form');
  $info = urldecode($info);
  if ($info != 'none' && $info && ($sched = explode('|', $info)) && $_POST['ctools_changed']) {
    $schedule_id = $sched[0];
    $page_index = $sched[1];
    $job_type = $_POST['ctools_changed'];    
    
    $type_handler = datasync_load_job_type((string)$job_type);
    //error out if schedule_id is not a digit, or we got an invalid job type or page_index is not a digit
    if (!ctype_digit($schedule_id) || !$type_handler || !ctype_digit($page_index)) {
      return;
    }
    
    $row = db_fetch_object(db_query("SELECT * FROM {datasync_scheduler_additions} WHERE schedule_id=%d", $schedule_id));
    if (!$row) {
      return;
    }

    $class = "datasync-schedule-$schedule_id";
    $form_state = array('rebuild' => TRUE, 'args' => array($schedule_id, $row, $job_type));
    $form = ctools_build_form("datasync_schedule_job_form_$page_index", $form_state);
  }
  else {
    $class = 'datasync-schedule-new';
    $job_type = NULL;
    $type_handler = datasync_load_job_type((string)$_POST['ctools_changed']);
    if ($type_handler) {
      $job_type = (string)$_POST['ctools_changed'];
    }
    $form_state = array('rebuild' => TRUE, 'args' => array(0, NULL, NULL, $job_type));
    $form = ctools_build_form('datasync_schedule_job_form', $form_state);
  }

  $commands = array();
  $commands[] = ctools_ajax_command_replace(".$class", $form);
  ctools_ajax_render($commands);
}

function datasync_schedule_job_form_delete($form, $form_state) {
  db_query("DELETE FROM {datasync_scheduler_additions} WHERE schedule_id=%d", $form['#schedule_id']);
  drupal_set_message(t('The scheduled job has been deleted.'));
}

function theme_datasync_schedule_job_form($form) {
  //we want a custom theme function for some elements
  foreach (array('job_type', 'days', 'hours', 'minutes', 'seconds') as $name) {
    $form[$name]['#theme'] = 'datasync_scheduler_bare_select';
  }
  $output = '';
  $output .= '<div class="scheduled-job-form">';
  $output .= t('Schedule a job of type') . drupal_render($form['job_type']);
  $output .= ' ' . t('every') . ' ';
  $output .= drupal_render($form['days']) . t('days');
  $output .= drupal_render($form['hours']) . t('hours');
  $output .= drupal_render($form['minutes']) . t('minutes');
  $output .= drupal_render($form['seconds']) . t('seconds');
  $output .= '<div class="clear-block"> </div>';
  if (isset($form['#last_add'])) {
    $output .= '<div class="datasync-last-added">' . $form['#last_add'] . '</div>';
  }
  $output .= drupal_render($form);
  $output .= '</div>';
  return $output;
}

/*
 * Portions ripped from theme_select and theme_form_element then modified
 */
function theme_datasync_scheduler_bare_select ($element) {
  $select = '';
  $size = $element['#size'] ? ' size="'. $element['#size'] .'"' : '';
  _form_set_class($element, array('form-select'));
  $multiple = $element['#multiple'];
  $value = '<select name="'. $element['#name'] .''. ($multiple ? '[]' : '') .'"'. ($multiple ? ' multiple="multiple" ' : '') . drupal_attributes($element['#attributes']) .' id="'. $element['#id'] .'" '. $size .'>'. form_select_options($element) .'</select>';
  
  // This is also used in the installer, pre-database setup.
  $t = get_t();

  $output = '<div class="form-item datasync-bare-select"';
  if (!empty($element['#id'])) {
    $output .= ' id="'. $element['#id'] .'-wrapper"';
  }
  $output .= " >\n";
  $required = !empty($element['#required']) ? '<span class="form-required" title="'. $t('This field is required.') .'">*</span>' : '';

  $output .= ' ' . $value . "\n";

  $output .= "</div>\n";

  return $output;
}

function datasync_timeout_page() {
  drupal_add_css(drupal_get_path('module', 'datasync_scheduler') . '/datasync_scheduler.css');
  $output = '';
  $output .= '<div id="datasync_timeout_page">';
  $output .= '<h2>' . t('Add a job timeout') . '</h2>';
  $output .= drupal_get_form('datasync_job_timeout_form');
  
  $output .= '<h2>' . t('Current job timeouts') . '</h2>';
  //form function supports retrieving all data in one query
  $result = db_query("SELECT * FROM {datasync_scheduler_timeouts} ORDER BY job_type ASC");
  $i = 0;
  while ($row = db_fetch_object($result)) {
    $output .= drupal_get_form('datasync_job_timeout_form_' . $i, $row->job_type);
    $i++;
  }
  $output .= '</div>';
  return $output;
}

function datasync_job_timeout_form($form_state, $job_type = NULL) {
  $form = array();
  $form['#theme'] = 'datasync_job_timeout_form';
  
  //If we have default values for this scheduled addition, set them
  $defaults = array();
  if ($job_type != NULL) {
    $result = db_query("SELECT * FROM {datasync_scheduler_timeouts} WHERE job_type='%s'", $job_type);
    $defaults = db_fetch_array($result);
 
    //convert days/hours/minutes/seconds select boxes into one interval integer
    $defaults['days'] = (int)($defaults['timeout_interval'] / 86400); //86400 == seconds in 1 day
    $defaults['hours'] = (int)(($defaults['timeout_interval'] % 86400) / 3600); // 3600 == seconds in 1 hour
    $defaults['minutes'] = (int)(($defaults['timeout_interval'] % 3600) / 60); // 60 == seconds in 1 minute
    $defaults['seconds'] = (int)($defaults['timeout_interval'] % 60);
    
    //button text
    $defaults['submit'] = t('Save');
    $defaults['delete'] = t('Delete');
    
    //let the submit handler know this is an update
    $form['#this_is_timeout_update'] = TRUE;
  }

  $options = array();
  foreach (datasync_job_types(TRUE) as $key => $def) {
    $options[$key] = $def['human_name'];
  }
  $form['job_type'] = array(
    '#title' => t('Job type'),
    '#type' => 'select',
    '#options' => $options,
    '#description' => t('Type of job you want to add'),
    '#default_value' => $defaults['job_type'] ? $defaults['job_type'] : FALSE,
  );
  
  $form['days'] = array(
    '#title' => t('Days'),
    '#type' => 'select',
    '#options' => drupal_map_assoc(range(0, 365)),
    '#default_value' => $defaults['days'] ? $defaults['days'] : FALSE,
  );
  $form['hours'] = array(
    '#title' => t('Hours'),
    '#type' => 'select',
    '#options' => drupal_map_assoc(range(0, 23)),
    '#default_value' => $defaults['hours'] ? $defaults['hours'] : FALSE,
  );
  $form['minutes'] = array(
    '#title' => t('Minutes'),
    '#type' => 'select',
    '#options' => drupal_map_assoc(range(0, 59)),
    '#default_value' => $defaults['minutes'] ? $defaults['minutes'] : FALSE,
  );
  $form['seconds'] = array(
    '#title' => t('Seconds'),
    '#type' => 'select',
    '#options' => drupal_map_assoc(range(0, 59)),
    '#default_value' => $defaults['seconds'] ? $defaults['seconds'] : FALSE,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => $defaults['submit'] ? $defaults['submit'] : t('Add job timeout'),
    '#submit' => array('datasync_job_timeout_form_submit'),
    '#validate' => array('datasync_job_timeout_form_validate'),
  );
  
  if ($defaults['delete']) {
    $form['delete'] = array(
      '#type' => 'submit',
      '#value' => $defaults['delete'],
      '#submit' => array('datasync_job_timeout_form_delete'),
    );
  }
  return $form;
}

function datasync_job_timeout_form_validate($form, $form_state) {
  $valid_interval = datasync_scheduler_interval_validate($form, $form_state);
  if (!$valid_interval) {
    form_set_error('', t("You must select some interval for timeouts (can't all be 0)"));
  }
  if (!$form['#this_is_timeout_update']) {
    $result = db_result(db_query("SELECT job_type FROM {datasync_scheduler_timeouts} WHERE job_type='%s'", $form_state['values']['job_type']));
    if ($result) {
      form_set_error('job_type', t("You cannot have more than one timeout of the same job type."));
    }
  }
}

function datasync_job_timeout_form_submit($form, $form_state) {
  $values = $form_state['values'];
  $timeout_interval = $values['days'] * 86400 + $values['hours'] * 3600 + $values['minutes'] * 60 + $values['seconds'];

  $timeout = array(
    'job_type' => $values['job_type'],
    'timeout_interval' => $timeout_interval,
  );
  
  if ($form['#this_is_timeout_update']) {
    drupal_write_record('datasync_scheduler_timeouts', $timeout, 'job_type');
  }
  else {
    drupal_write_record('datasync_scheduler_timeouts', $timeout);
  }
}

function datasync_job_timeout_form_delete($form, $form_state) {
  db_query("DELETE FROM {datasync_scheduler_timeouts} WHERE job_type='%s'", $form_stat['values']['job_type']);
}

function theme_datasync_job_timeout_form($form) {
  //we want a custom theme function for some elements
  foreach (array('job_type', 'days', 'hours', 'minutes', 'seconds') as $name) {
    $form[$name]['#theme'] = 'datasync_scheduler_bare_select';
  }
  $output = '';
  $output .= '<div class="job-timeout-form">';
  $output .= t('Timeout jobs of type') . drupal_render($form['job_type']);
  $output .= ' ' . t('if they have not completed after') . ' ';
  $output .= drupal_render($form['days']) . t('days');
  $output .= drupal_render($form['hours']) . t('hours');
  $output .= drupal_render($form['minutes']) . t('minutes');
  $output .= drupal_render($form['seconds']) . t('seconds');
  $output .= '<div class="clear-block"> </div>';
  $output .= drupal_render($form);
  $output .= '</div>';
  return $output;
}


/**
 * Used by both scheduled additions and timeouts
 * @param $form
 * @param $form_state
 * @return 
 *   Returns TRUE on valid interval, FALSE on invalid interval
 */
function datasync_scheduler_interval_validate($form, $form_state) {
  $values = $form_state['values'];
  if (!$values['days'] && !$values['hours'] && !$values['minutes'] && !$values['seconds']) {
    return FALSE;
  }
  return TRUE;
}
