<?php

/**
 * Filter by job status
 */
class datasync_handler_filter_status extends views_handler_filter_in_operator {
  function get_value_options() {
    if (!isset($this->value_options)) {
      $this->value_title = t('Job status');
      $options = array(
        'added' => t('Added'),
        'queued' => t('Queued'),
        'done' => t('Done'),
        'running' => t('Running'),
        'phase_completed' => t('Phase completed'),
      ); 
      $this->value_options = $options;
    }
  }
  
  function operator_options() {
    return array(
      'in' => t('Is one of'),
    );
  }
  
  function query() {
    if (empty($this->value)) {
      return;
    }
    $table = $this->ensure_my_table();
    $placeholder = !empty($this->definition['numeric']) ? '%d' : "'%s'";
    
    $subs = array();
    foreach ($this->value as $status) {
      $sub = '';
      switch ($status) {
        case 'added':
          $sub .= "$table.started=0 AND $table.completed!=0 AND $table.phase='datasync_added'";
          break;
        case 'queued':
          $sub .= "$table.started=0 AND $table.phase!='datasync_done'";
          break;
        case 'done':
          $sub .= "$table.started=0 AND $table.phase='datasync_done'";
          break;
        case 'running':
          $sub .= "$table.started!=0 AND $table.completed=0";
          break;
        case 'phase_completed':
          $sub .= "$table.started!=0 AND $table.completed!=0";
          break;
      }
      if (strlen($sub)) {
        $subs[] = $sub;
      }
    }
    
    if (!count($subs)) {
      return;
    }
    
    $where = ' ( (' . implode(") OR (", $subs) . ') )';


    $this->query->add_where($this->options['group'], $where);
  }
}
